#########################################################################/**
# @RdocPackage R.methodsS3
#
# \description{
#  @eval "packageDescription('R.methodsS3')$Description"
#  This contents of this package was extracted from the 
#  \pkg{R.oo} package [1].
# }
#
# \section{Installation and updates}{
#   To install this package do\cr
#
#   \code{install.packages("R.methodsS3")}
#
#   To get the "devel" version, see \url{http://www.braju.com/R/}.
# }
#
# \section{Dependancies and other requirements}{
#   This package only requires a standard \R installation.
# }
#
# \section{To get started}{
#   To get started, see:
#   \enumerate{
#     \item @see "setMethodS3" - Simple and safe creation of S3 methods
#         and, whenever needed,  automatic creation of S3 generic function.
#   }
# } 
#
# \section{Further readings}{
#   For a detailed introduction to the package, see [1].
# }
#
# \section{How to cite this package}{
#   Whenever using this package, please cite [1] as\cr
#
#   @howtocite "R.methodsS3"
# }
#
# @author
#
# \section{License}{
#   The releases of this package is licensed under 
#   LGPL version 2.1 or newer.
# }
#
# \references{
#  [1] @include "../incl/BengtssonH_2003.bib.Rdoc" \cr
# }
#*/#########################################################################  
